'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ATTRIBUTES_TO_PROPS = exports.ATTRIBUTES = exports.FILTER_CAST_BOOL = exports.FILTER_CAST_NUMBER = exports.FILTER_DENY = exports.FILTER_ALLOW = exports.TAGS_BLACKLIST = exports.TAGS = exports.CONFIG_BLOCK = exports.CONFIG_INLINE = exports.TYPE_INLINE_BLOCK = exports.TYPE_BLOCK = exports.TYPE_INLINE = exports.PARSER_DENY = exports.PARSER_ALLOW = exports.EMAIL_PATTERN = exports.EMAIL_USERNAME_PATTERN = exports.EMAIL_CLASS_PART = exports.IP_PATTERN = exports.IP_V4_PATTERN = exports.IP_V4_PART = exports.URL_PATTERN = exports.URL_FRAGMENT_PATTERN = exports.URL_QUERY_PATTERN = exports.URL_PATH_PATTERN = exports.URL_PORT_PATTERN = exports.URL_HOST_PATTERN = exports.URL_AUTH_PATTERN = exports.URL_SCHEME_PATTERN = exports.URL_CHAR_PART = exports.HASHTAG_PATTERN = exports.ALNUM_CHAR = undefined;

var _freeze = require('babel-runtime/core-js/object/freeze');

var _freeze2 = _interopRequireDefault(_freeze);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @copyright   2016, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 * 
 */

/* eslint-disable no-useless-escape, max-len */

// https://blog.codinghorror.com/the-problem-with-urls/
// http://www.regular-expressions.info/email.html

/*:: import type { NodeConfig } from './types';*/
/*:: type ConfigMap = { [key: string]: NodeConfig };*/


// This pattern will be used at the start or end of other patterns,
// making it easier to apply matches without capturing special chars.
/*:: type FilterMap = { [key: string]: number };*/
var ALNUM_CHAR /*: string*/ = exports.ALNUM_CHAR = '[a-z0-9]{1}';

var HASHTAG_PATTERN /*: string*/ = exports.HASHTAG_PATTERN = '#([-a-z0-9_]+)';

var URL_CHAR_PART /*: string*/ = exports.URL_CHAR_PART = 'a-z0-9-_~%!$&\'*+;:@'; // Disallow . , ( )
var URL_SCHEME_PATTERN /*: string*/ = exports.URL_SCHEME_PATTERN = '(https?://)?';
var URL_AUTH_PATTERN /*: string*/ = exports.URL_AUTH_PATTERN = '([' + URL_CHAR_PART + ']+@)?';
var URL_HOST_PATTERN /*: string*/ = exports.URL_HOST_PATTERN = '((?:www\\.)?' + ALNUM_CHAR + '[-a-z0-9.]*[-a-z0-9]+\\.[a-z]{2,63})';
var URL_PORT_PATTERN /*: string*/ = exports.URL_PORT_PATTERN = '(:[0-9]+)?';
var URL_PATH_PATTERN /*: string*/ = exports.URL_PATH_PATTERN = '(/[' + URL_CHAR_PART + '/]*(?:\\.[a-z]{2,8})?)?';
var URL_QUERY_PATTERN /*: string*/ = exports.URL_QUERY_PATTERN = '(\\?[' + URL_CHAR_PART + '=\\[\\]/\\\\]*)?';
var URL_FRAGMENT_PATTERN /*: string*/ = exports.URL_FRAGMENT_PATTERN = '(#[\\w%/]*)?';
var URL_PATTERN /*: string*/ = exports.URL_PATTERN = [URL_SCHEME_PATTERN, URL_AUTH_PATTERN, URL_HOST_PATTERN, URL_PORT_PATTERN, URL_PATH_PATTERN, URL_QUERY_PATTERN, URL_FRAGMENT_PATTERN].join('');

var IP_V4_PART /*: string*/ = exports.IP_V4_PART = '(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)';
var IP_V4_PATTERN /*: string*/ = exports.IP_V4_PATTERN = '(' + IP_V4_PART + '\\.' + IP_V4_PART + '\\.' + IP_V4_PART + '\\.' + IP_V4_PART + ')';
var IP_PATTERN /*: string*/ = exports.IP_PATTERN = [URL_SCHEME_PATTERN, URL_AUTH_PATTERN, IP_V4_PATTERN, URL_PORT_PATTERN, URL_PATH_PATTERN, URL_QUERY_PATTERN, URL_FRAGMENT_PATTERN].join('');

var EMAIL_CLASS_PART /*: string*/ = exports.EMAIL_CLASS_PART = '[a-z0-9!#$%&?*+=_{|}~-]+';
var EMAIL_USERNAME_PATTERN /*: string*/ = exports.EMAIL_USERNAME_PATTERN = '(' + ALNUM_CHAR + EMAIL_CLASS_PART + '(?:\\.' + EMAIL_CLASS_PART + ')*' + ALNUM_CHAR + ')';
var EMAIL_PATTERN /*: string*/ = exports.EMAIL_PATTERN = EMAIL_USERNAME_PATTERN + '@' + URL_HOST_PATTERN;

// Parser rules for HTML tags
var PARSER_ALLOW /*: number*/ = exports.PARSER_ALLOW = 1; // Allow element and children
var PARSER_DENY /*: number*/ = exports.PARSER_DENY = 2; // Do not render this element or its children

var TYPE_INLINE /*: string*/ = exports.TYPE_INLINE = 'inline';
var TYPE_BLOCK /*: string*/ = exports.TYPE_BLOCK = 'block';
var TYPE_INLINE_BLOCK /*: string*/ = exports.TYPE_INLINE_BLOCK = 'inline-block'; // Special case

var CONFIG_INLINE /*: NodeConfig*/ = exports.CONFIG_INLINE = {
  rule: PARSER_ALLOW,
  type: TYPE_INLINE,
  inline: true,
  block: false,
  self: false,
  void: false,
  parent: [],
  children: []
};

var CONFIG_BLOCK /*: NodeConfig*/ = exports.CONFIG_BLOCK = {
  rule: PARSER_ALLOW,
  type: TYPE_BLOCK,
  inline: true,
  block: true,
  self: true,
  void: false,
  parent: [],
  children: []
};

// Tags not listed here will be marked as pass-through
// https://developer.mozilla.org/en-US/docs/Web/HTML/Element
var tagConfigs = {
  a: {
    type: TYPE_INLINE_BLOCK,
    block: true
  },
  address: {
    self: false
  },
  audio: {
    children: ['track', 'source']
  },
  br: {
    inline: false,
    void: true
  },
  button: {
    type: TYPE_INLINE_BLOCK
  },
  dd: {
    parent: ['dl']
  },
  dl: {
    children: ['dt']
  },
  dt: {
    parent: ['dl'],
    children: ['dd']
  },
  figcaption: {
    parent: ['figure']
  },
  footer: {
    self: false
  },
  header: {
    self: false
  },
  h1: {
    self: false
  },
  h2: {
    self: false
  },
  h3: {
    self: false
  },
  h4: {
    self: false
  },
  h5: {
    self: false
  },
  h6: {
    self: false
  },
  hr: {
    inline: false,
    block: false,
    void: true
  },
  img: {
    inline: false,
    void: true
  },
  li: {
    self: false,
    parent: ['ul', 'ol']
  },
  main: {
    self: false
  },
  ol: {
    children: ['li']
  },
  picture: {
    children: ['source', 'img']
  },
  source: {
    inline: false,
    parent: ['audio', 'video', 'picture'],
    void: true
  },
  span: {
    self: true
  },
  table: {
    children: ['thead', 'tbody', 'tfoot', 'tr']
  },
  tbody: {
    parent: ['table'],
    children: ['tr']
  },
  td: {
    parent: ['tr']
  },
  tfoot: {
    parent: ['table'],
    children: ['tr']
  },
  th: {
    parent: ['tr']
  },
  thead: {
    parent: ['table'],
    children: ['tr']
  },
  tr: {
    parent: ['table', 'tbody', 'thead', 'tfoot'],
    children: ['th', 'td']
  },
  track: {
    inline: false,
    parent: ['audio', 'video'],
    void: true
  },
  ul: {
    children: ['li']
  },
  video: {
    children: ['track', 'source']
  }
};

function createConfigBuilder(config /*: NodeConfig*/) {
  return function (tagName /*: string*/) {
    tagConfigs[tagName] = (0, _extends3.default)({}, config, tagConfigs[tagName] || {});
  };
}

// Add inline tags
['a', 'abbr', 'b', 'br', 'button', 'cite', 'code', 'del', 'dfn', 'em', 'i', 'img', 'ins', 'kbd', 'label', 'mark', 'output', 'picture', 'q', 's', 'samp', 'source', 'span', 'strong', 'sub', 'sup', 'time', 'track', 'u', 'var', 'video'].forEach(createConfigBuilder(CONFIG_INLINE));

// Add block tags
['address', 'article', 'aside', 'audio', 'blockquote', 'dd', 'details', 'div', 'dl', 'dt', 'fieldset', 'figcaption', 'figure', 'footer', 'header', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'hr', 'legend', 'li', 'main', 'nav', 'ol', 'p', 'pre', 'section', 'summary', 'table', 'thead', 'tbody', 'tfoot', 'tr', 'th', 'td', 'ul'].forEach(createConfigBuilder(CONFIG_BLOCK));

// Disable this map from being modified
var TAGS /*: ConfigMap*/ = exports.TAGS = (0, _freeze2.default)(tagConfigs);

// Tags that should never be allowed, even if the whitelist is disabled
var TAGS_BLACKLIST /*: { [tagName: string]: boolean }*/ = exports.TAGS_BLACKLIST = {
  applet: true,
  body: true,
  canvas: true,
  embed: true,
  frame: true,
  frameset: true,
  head: true,
  html: true,
  iframe: true,
  object: true,
  script: true,
  style: true
};

// Filters apply to HTML attributes
var FILTER_ALLOW /*: number*/ = exports.FILTER_ALLOW = 1;
var FILTER_DENY /*: number*/ = exports.FILTER_DENY = 2;
var FILTER_CAST_NUMBER /*: number*/ = exports.FILTER_CAST_NUMBER = 3;
var FILTER_CAST_BOOL /*: number*/ = exports.FILTER_CAST_BOOL = 4;

// Attributes not listed here will be denied
// https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes
var ATTRIBUTES /*: FilterMap*/ = exports.ATTRIBUTES = (0, _freeze2.default)({
  alt: FILTER_ALLOW,
  cite: FILTER_ALLOW,
  class: FILTER_ALLOW,
  colspan: FILTER_CAST_NUMBER,
  controls: FILTER_CAST_BOOL,
  datetime: FILTER_ALLOW,
  default: FILTER_CAST_BOOL,
  disabled: FILTER_CAST_BOOL,
  dir: FILTER_ALLOW,
  height: FILTER_ALLOW,
  href: FILTER_ALLOW,
  id: FILTER_ALLOW,
  kind: FILTER_ALLOW,
  label: FILTER_ALLOW,
  lang: FILTER_ALLOW,
  loop: FILTER_CAST_BOOL,
  muted: FILTER_CAST_BOOL,
  poster: FILTER_ALLOW,
  role: FILTER_ALLOW,
  rowspan: FILTER_CAST_NUMBER,
  span: FILTER_CAST_NUMBER,
  src: FILTER_ALLOW,
  target: FILTER_ALLOW,
  title: FILTER_ALLOW,
  width: FILTER_ALLOW
});

// Attributes to camel case for React props
var ATTRIBUTES_TO_PROPS /*: { [key: string]: string }*/ = exports.ATTRIBUTES_TO_PROPS = (0, _freeze2.default)({
  class: 'className',
  colspan: 'colSpan',
  datetime: 'dateTime',
  rowspan: 'rowSpan'
});